//
//  MacounProtocol.m
//  Webviews
//
//  Created by Clemens Wagner on 15.08.15.
//  Copyright (c) 2015 Clemens Wagner. All rights reserved.
//

#import "MacounProtocol.h"

static NSString * const kMacounProtocolSkip = @"kMacounProtocolSkip";

@interface MacounProtocol ()

@property (strong, nonatomic) NSURLSessionTask *task;

@end

@implementation MacounProtocol

+ (BOOL)canInitWithRequest:(NSURLRequest *)inRequest {
    NSURL *theURL = [inRequest URL];
    id isSkipped = [self propertyForKey:kMacounProtocolSkip inRequest:inRequest];
    
    return ![isSkipped boolValue] && [theURL.path isEqualToString:@"/autocompletion"];
}

+ (NSURLRequest *)canonicalRequestForRequest:(NSURLRequest *)inRequest {
    NSMutableURLRequest *theRequest = [inRequest mutableCopy];

    [NSURLProtocol setProperty:@YES forKey:kMacounProtocolSkip inRequest:theRequest];
    return [theRequest copy];
}

- (void)startLoading {
    NSURLSession *theSession = [NSURLSession sharedSession];
    
self.task = [theSession dataTaskWithRequest:self.request completionHandler:^(NSData *inData, NSURLResponse *inResponse, NSError *inError) {
    if(inData == nil) {
        [self.client URLProtocol:self didFailWithError:inError];
    }
    else {
        NSData *theData = [self processData:inData];
        
        [self.client URLProtocol:self didReceiveResponse:inResponse cacheStoragePolicy:NSURLCacheStorageNotAllowed];
        [self.client URLProtocol:self didLoadData:theData];
        [self.client URLProtocolDidFinishLoading:self];
    }
}];
    [self.task resume];
}

- (void)stopLoading {
    [self.task cancel];
    self.task = nil;
}

- (NSData *)processData:(NSData *)inData {
    NSError *theError = nil;
    NSArray *theItems = [NSJSONSerialization JSONObjectWithData:inData options:0 error:&theError];
    NSData *theResult = inData;
    
    if(theItems == nil) {
        NSLog(@"error = %@", theError);
    }
    else {
        NSInteger theIndex = [theItems indexOfObject:@"Macoun"];
        
        if(theIndex != NSNotFound) {
            NSMutableArray *theNewItems = [NSMutableArray arrayWithCapacity:theItems.count];
            
            [theNewItems addObject:@{@"label": @"* Macoun", @"value": @"Macoun" }];
            [theItems enumerateObjectsUsingBlock:^(NSString *inValue, NSUInteger inIndex, BOOL *outStop) {
                if(inIndex != theIndex) {
                    [theNewItems addObject:@{ @"label": inValue, @"value": inValue}];
                }
            }];
            theResult = [NSJSONSerialization dataWithJSONObject:theNewItems options:0 error:NULL];
        }
    }
    return theResult;
}

@end
