#!/usr/bin/python

import bisect
import getopt
import json
import logging
import sys
import urlparse

from xml.sax.saxutils import escape

import SimpleHTTPServer
import SocketServer

configuration = {
    '-f': 'data.json',
    '-s': '20'
}
options, arguments = getopt.getopt(sys.argv[1:], 'f:s:');
configuration.update(dict(options))
kSize = int(configuration['-s'])
kData = None
kKeys = None

class Handler(SimpleHTTPServer.SimpleHTTPRequestHandler):
    def optionalParameter(self, inParameters, inName):
        if inName in inParameters:
            theValues = inParameters[inName]
            return theValues[0].decode('UTF-8') if len(theValues) > 0 else None
        else:
            return None
            
    def search(self, inComponents):
        theParameters = urlparse.parse_qs(inComponents.query)
        theTerm = self.optionalParameter(theParameters, 'term')
        theResult = {}
        if theTerm:
            theTerm = theTerm.lower()
            theIndex = bisect.bisect_left(kKeys, theTerm)
            if theIndex != len(kKeys) and kKeys[theIndex] == theTerm:
                theResult = kData[theIndex][1]
        self.send_response(200)
        self.send_header('Content-Type', 'text/json')
        self.end_headers()
        self.wfile.write(json.dumps(theResult).encode('UTF-8'))        
    
    def autocompletion(self, inComponents):
        theParameters = urlparse.parse_qs(inComponents.query)
        theTerm = self.optionalParameter(theParameters, 'term')
        logging.info("autocompletion: term = %s" % theTerm)
        theResult = []
        if theTerm:
            theTerm = theTerm.lower()
            theIndex = bisect.bisect_left(kKeys, theTerm)
            while kKeys[theIndex].startswith(theTerm) and len(theResult) < kSize:
                theResult.append(kData[theIndex][0])
                theIndex += 1
        self.send_response(200)
        self.send_header('Content-Type', 'text/json')
        self.end_headers()
        self.wfile.write(json.dumps(theResult).encode('UTF-8'))        
        
    def callback(self):
        theResult = None
        theComponents = urlparse.urlparse(self.path)
        thePath = theComponents.path
        if len(thePath) > 0:
            theParts = thePath[1:].split('/', 1)
            theCallback = getattr(self, theParts[0], None)
            if theCallback:
                theResult = (theCallback, theComponents)
        return theResult

    def do_GET(self):
        theCall = self.callback()
        if theCall:
            return theCall[0](theCall[1])
        else:
            return SimpleHTTPServer.SimpleHTTPRequestHandler.do_GET(self)

with open(configuration['-f'], 'r') as theStream:
    kData = json.load(theStream)
    kKeys = [t[0].lower() for t in kData]

theHandler = Handler
theServer = SocketServer.TCPServer(('0.0.0.0', 8080), theHandler)

theServer.serve_forever()
